using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL.GUI.Seekers;
using PhoenixDLL.Class.Managers;
using PhoenixDLL.GUI;
using Utils.DataBase;
using PhoenixDLL.Class;
using System.IO;
using System.Data.OleDb;
using System.Drawing.Imaging;
using System.Threading;
using Utils.Logs;
using Utils.ImageProcessing;

namespace PhoenixPics.GUI
{
    /// <summary>
    /// Gestion des photos
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// Manager SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        /// <summary>
        /// Gestionnaire de Patient
        /// </summary>
        private PatientManager pat = new PatientManager();
        /// <summary>
        /// Indicates whether the frame is loading
        /// </summary>
        private bool vIsLoading = true;
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
            tsMenu_Options.Visible = Options.Instance.OptionsMenuVisible;
            ts_Menu.Visible = Options.Instance.PotoMenu.IsVisible;
            tsMenu_Visible.Checked = Options.Instance.PotoMenu.IsVisible;
            SetToolStrips();
            Options.Instance.OptionChanged += new OnOptionChangedHandler(Event_OptionChanged);

            lbl_Change.Dock = DockStyle.None;
            lbl_Change.Location = new Point(9999, 9999);
            tlp_General.Location = new Point(0, 0);
            tlp_General.Dock = DockStyle.Fill;
        }
        #endregion

        #region Events
        /// <summary>
        /// Clique sur la selection d'un patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SelectionPatient(object sender, EventArgs e)
        {
            #region Selection du patient
            try
            {
                this.pb_Picture.Image = null;
                stl_NomPatient.Text = "Aucun patient";
                fr_PatientSeeker fr = new fr_PatientSeeker();
                DialogResult dr = fr.ShowDialog();
                if (dr == DialogResult.OK)
                {
                    pat = fr.SelectedItem;
                    il_ThumbList.Images.Clear();
                    stl_NomPatient.Text = pat.ToString();
                    lv_Thumb.Items.Clear();

                    lbl_Change.Dock = DockStyle.None;
                    lbl_Change.Location = new Point(9999, 9999);
                    tlp_General.Location = new Point(0, 0);
                    tlp_General.Dock = DockStyle.Fill;

                    #region Select the thumbnails
                    RefreshPics();
                    #endregion
                }
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, this, LogType.Error);
                MessageBox.Show(ex.Message, "Attention", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            #endregion
        }
        /// <summary>
        /// Choix des vOptions
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Options(object sender, EventArgs e)
        {
            fr_Options fr = new fr_Options();
            fr.ShowDialog();
            SqlManager.DbPath = Options.GetInstance().Path;
        }
        /// <summary>
        /// Ajout de photos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddPic(object sender, EventArgs e)
        {
            if (pat.IDPatient == -1) return;

            DialogResult dr = ofd_PicturesSelection.ShowDialog();

            if (dr == DialogResult.OK)
            {
                tspb_ProgressBar.Visible = true;
                Image bmp;
                MemoryStream ms;
                SqlManager.Open();
                tspb_ProgressBar.Maximum = ofd_PicturesSelection.FileNames.Length * 5;
                tspb_ProgressBar.Minimum = 0;
                tspb_ProgressBar.Step = 1;
                il_ThumbList = lv_Thumb.LargeImageList;
                try
                {
                    foreach (string file in ofd_PicturesSelection.FileNames)
                    {
                        ms = new MemoryStream();
                        SqlManager.PrepareProcedure("INSERT_Photo", false);
                        SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = pat.IDPatient;
                        tspb_ProgressBar.PerformStep();

                        bmp = new Bitmap(file);
                        bmp.Save(ms, ImageFormat.Jpeg);
                        SqlManager.Parameters.Add("[_Pic]", OleDbType.Binary).Value = ms.ToArray();
                        tspb_ProgressBar.PerformStep();

                        bmp = ImageResizer.Resize((Bitmap)bmp, 10);
                        bmp.Save(ms, ImageFormat.Jpeg);
                        SqlManager.Parameters.Add("[_Thumb]", OleDbType.Binary).Value = ms.ToArray();
                        tspb_ProgressBar.PerformStep();

                        string rmq = "";
                        if (ofd_PicturesSelection.FileNames.Length <= 1)
                        {
                            fr_Remarks fr = new fr_Remarks();
                            fr.Image = bmp;
                            DialogResult drr = fr.ShowDialog();
                            rmq = fr.Remark;
                        }

                        SqlManager.Parameters.Add("[_Rmq]", OleDbType.Variant).Value = rmq;
                        tspb_ProgressBar.PerformStep();

                        object LastId = 0;
                        SqlManager.ExecuteNonQueryProcedure(ref LastId);
                        SqlManager.Parameters.Clear();
                        il_ThumbList.Images.Add(bmp);

                        lv_Thumb.Items.Add(LastId.ToString()).ImageIndex = il_ThumbList.Images.Count - 1; ;
                        tspb_ProgressBar.PerformStep();
                    }

                }
                catch (Exception ex)
                {
                    Log.Instance.Write(ex.Message, this, LogType.Error);
                    MessageBox.Show("Une erreur est survenue lors de l'ajout d'une photo.\r\n" + ex.Message,
                        "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                lv_Thumb.LargeImageList = il_ThumbList;
                SqlManager.Close();
                tspb_ProgressBar.Value = 0;
                tspb_ProgressBar.Visible = false;
            }
        }
        /// <summary>
        /// Evnement de demande de redessinage de la ListBox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_ListBoxDrawItem(object sender, DrawItemEventArgs e)
        {
            try
            {
                if (e.Index < 0 || il_ThumbList.Images.Count <= e.Index) return;
                Point p = e.Bounds.Location;
                il_ThumbList.Draw(e.Graphics, p, e.Index);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur innatendue est survenue! (" + ex.Message + ")",
                    "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
        }
        /// <summary>
        /// On  slectionne une image dans la ThumbBox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_ListBoxIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (lv_Thumb.SelectedItems.Count <= 0) return;
                SqlManager.SqlQuery = "SELECT * FROM photo WHERE id_photo = " + lv_Thumb.SelectedItems[0].Text;
                DataSet ds = new DataSet();
                SqlManager.ExecuteSql(ref ds);

                pb_Picture.Image = new Bitmap(new MemoryStream((byte[])ds.Tables[0].Rows[0]["BinPic"]));
                tb_Remarks.Text = (string)ds.Tables[0].Rows[0]["Remarques"];
                btn_Del.Enabled = true;
                btn_Modify.Enabled = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est survenue!", "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
            #region
            /*if (lv_Thumb.SelectedIndex == -1) return;
            int idp = Int32.Parse(lv_Thumb.Items[lv_Thumb.SelectedIndex].ToString());

            DataSet ds = new DataSet();
            int i = SqlManager.ExecuteSql("SELECT * FROM PHOTO WHERE id_Photo = " + idp, ref ds);

            Bitmap bmp = new Bitmap(new MemoryStream((byte[])ds.Tables[0].Rows[0]["BinPic"]));
            string Rmq = (string)ds.Tables[0].Rows[0]["Remarques"];
            pb_Picture.Image = bmp;
            tb_Remarks.Text = Rmq;
            btn_Del.Enabled = btn_Modify.Enabled = true;*/
            #endregion
        }
        /// <summary>
        /// Double clique sur la PictureBox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DoubleClick_PictureBox(object sender, EventArgs e)
        {
            if (pb_Picture.Image == null) return;

            fr_FullScreenPicture fr = new fr_FullScreenPicture();
            fr.Image = pb_Picture.Image;
            fr.ShowDialog();
        }
        /// <summary>
        /// Bouton modifier une photo
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Modify(object sender, EventArgs e)
        {
            if (btn_Modify.Text.ToLower() == "Activer Modifier".ToLower())
            {
                tb_Remarks.ReadOnly = false;
                btn_Modify.Text = "Modifier";
            }
            else
                if (lv_Thumb.SelectedItems[0] != null)
                {
                    SqlManager.ExecuteNonQuery("UPDATE Photo SET Remarques = \""
                            + tb_Remarks.Text + "\" WHERE id_Photo = " + this.lv_Thumb.SelectedItems[0].Text);
                    tb_Remarks.ReadOnly = true;
                    btn_Modify.Enabled = false;
                    btn_Modify.Text = "Activer Modifier";
                }
        }
        /// <summary>
        /// Suppression d'une photo
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Del(object sender, EventArgs e)
        {
            try
            {
                DialogResult dr =
                    MessageBox.Show("Voulez-vous supprimer cette photo?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (dr == DialogResult.Yes)
                {
                    SqlManager.ExecuteNonQuery("DELETE * FROM Photo WHERE id_Photo = " + (string)this.lv_Thumb.SelectedItems[0].Text);
                    pb_Picture.Image = null;
                    lv_Thumb.Items.RemoveAt(this.lv_Thumb.SelectedItems[0].Index);
                    //il_ThumbList.Images.RemoveAt(this.lv_Thumb.SelectedItems[0].Index);
                    btn_Modify.Enabled = false;
                    btn_Del.Enabled = false;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Impossible de supprimer la photo!", "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
        }
        /// <summary>
        /// A propos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_APropos_Click(object sender, EventArgs e)
        {
            fr_About fr = new fr_About();
            fr.ShowDialog();
        }
        /// <summary>
        /// Sets the visibility of the menus
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_VisibleMenu(object sender, EventArgs e)
        {
            if (!tsMenu_Visible.Checked)
            {
                tsMenu_Visible.Checked = true;
                ts_Menu.Visible = true;
                Options.Instance.PotoMenu.IsVisible = true;
            }
            else
            {
                tsMenu_Visible.Checked = false;
                ts_Menu.Visible = false;
                Options.Instance.PotoMenu.IsVisible = false;
            }
        }
        /// <summary>
        /// Click on quit window
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Close(object sender, EventArgs e)
        {
            this.Close();
        }
        /// <summary>
        /// The options changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_OptionChanged(object sender, EventArgs e)
        {
            SetToolStrips();
        }
        /// <summary>
        /// The selected Index changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_ThumbSelectedItemChanged(object sender, EventArgs e)
        {
            try
            {
                if (lv_Thumb.SelectedItems.Count <= 0) return;
                SqlManager.SqlQuery = "SELECT * FROM photo WHERE id_photo = " + lv_Thumb.SelectedItems[0].Text;
                DataSet ds = new DataSet();
                SqlManager.ExecuteSql(ref ds);

                pb_Picture.Image = new Bitmap(new MemoryStream((byte[])ds.Tables[0].Rows[0]["BinPic"]));
                tb_Remarks.Text = (string)ds.Tables[0].Rows[0]["Remarques"];
                btn_Del.Enabled = true;
                btn_Modify.Enabled = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est survenue!", "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
        }
        /// <summary>
        /// Focus on the frame
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FrameEnter(object sender, EventArgs e)
        {
            if (!vIsLoading)
            {
                SqlManager.SqlQuery = "SELECT COUNT(*) FROM photo WHERE id_patient = " + pat.IDPatient + ";";
                int count = (int)SqlManager.ExecuteScalar();
                if (count != lv_Thumb.Items.Count)
                {
                    DialogResult dr = MessageBox.Show("Il y a eu modification des photos du patient. Voulez-vous recharger les photos?",
                        "Attention! ", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk);
                    if (dr == DialogResult.No)
                    {
                        tlp_General.Dock = DockStyle.None;
                        tlp_General.Location = new Point(9999, 9999);
                        lbl_Change.Text = "Les photos de ce patient on t modifies ailleur!";
                        lbl_Change.Location = new Point(0, 0);
                        lbl_Change.Dock = DockStyle.Fill;

                    }
                    if (dr == DialogResult.Yes)
                    {
                        RefreshPics();
                    }
                }
            }
        }
        /// <summary>
        /// The frame is loading
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FrameLoad(object sender, EventArgs e)
        {
            vIsLoading = false;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Sets the ToolStrips' image size
        /// </summary>
        private void SetToolStrips()
        {
            ts_Menu.AutoSize = false;
            ts_Menu.ImageScalingSize = Options.Instance.MenuImageScalingSize;
            ts_Menu.AutoSize = true;
        }
        /// <summary>
        /// Refresh the pics
        /// </summary>
        private void RefreshPics()
        {
            this.pb_Picture.Image = null;
            btn_Del.Enabled = false;
            lv_Thumb.Clear();
            il_ThumbList.Images.Clear();
            SeekPics sp = new SeekPics();
            sp.Patient = pat;
            sp.Frame = this;
            Thread th = new Thread(new ThreadStart(sp.Process));
            th.Name = "Picture thread";
            th.Start();
        }
        #endregion
    }

    /// <summary>
    /// Thread class the the seek of the pictures
    /// </summary>
    internal class SeekPics
    {
        #region Variables
        /// <summary>
        /// Contains the pics of the patient
        /// </summary>
        private DataSet vDSPics = new DataSet();
        /// <summary>
        /// Indicates the ID of the selected patient
        /// </summary>
        private PatientManager vPatient;
        /// <summary>
        /// Reference on the main frame
        /// </summary>
        private fr_Main vFrame;

        /// <summary>
        /// Delegate for the GUI treatment
        /// </summary>        
        private delegate void UpdateGuiHandler();
        /// <summary>
        /// Mthode de tratement inter threads
        /// </summary>
        private UpdateGuiHandler UpdateGui;
        /// <summary>
        /// Sql manager
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        /// <summary>
        /// Contains the thumbs of the pictures
        /// </summary>
        private ImageList vImageList = new ImageList();
        #endregion

        #region Constructors
        /// <summary>
        /// Contructor
        /// </summary>
        public SeekPics() { }
        #endregion

        #region Methods
        /// <summary>
        /// Thread treatment
        /// </summary>
        public void Process()
        {
            try
            {
                #region Seek pictures
                if (vPatient == null) return;
                vDSPics = new DataSet();
                SqlManager.SqlQuery = "SELECT id_photo, thumb FROM photo WHERE id_patient = " + vPatient.IDPatient + ";";
                SqlManager.ExecuteSql(ref vDSPics);
                #endregion

                #region Store the thumbs
                vImageList = new ImageList();
                vImageList.ImageSize = new Size(100, 100);
                vImageList.ColorDepth = ColorDepth.Depth24Bit;
                MemoryStream ms;
                Bitmap bmp;

                //Log.Instance.RestartWatch();
                foreach (DataRow dr in vDSPics.Tables[0].Rows)
                {
                    ms = new MemoryStream((byte[])dr["thumb"]);
                    bmp = (Bitmap)Bitmap.FromStream(ms, true, false);
                    //vImageList.Images.Add(bmp);
                    vFrame.lv_Thumb.LargeImageList.Images.Add(bmp);
                }
                //Log.Instance.StopWatch("Image Add in the Thread!");
                #endregion

                #region Display pics
                UpdateGui = new UpdateGuiHandler(this.RefreshGui);
                vFrame.Invoke(UpdateGui);
                #endregion
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
            }
            finally { vDSPics.Dispose(); }
        }
        /// <summary>
        /// Refreshes the GUI with the pictures
        /// </summary>
        private void RefreshGui()
        {
            try
            {
                int Max = vDSPics.Tables[0].Rows.Count;
                vFrame.tspb_ProgressBar.Maximum = vDSPics.Tables[0].Rows.Count + 2;
                vFrame.tspb_ProgressBar.Minimum = 0;
                vFrame.tspb_ProgressBar.Value = 1;
                vFrame.tspb_ProgressBar.Step = 1;
                //vFrame.lv_Thumb.LargeImageList = vImageList;
                vFrame.tspb_ProgressBar.PerformStep();

                for (int i = 0; i < vDSPics.Tables[0].Rows.Count; i++)
                {
                    vFrame.lv_Thumb.Items.Add(((int)vDSPics.Tables[0].Rows[i]["id_photo"]).ToString());
                    vFrame.lv_Thumb.Items[i].ImageIndex = i;
                    vFrame.tspb_ProgressBar.PerformStep();
                }
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                MessageBox.Show("Impossible d'afficher les photos du patient!", "Erreur",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally { vFrame.tspb_ProgressBar.Value = 0; }
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the ID of the selected patient
        /// </summary>
        public PatientManager Patient
        {
            get { return vPatient; }
            set { vPatient = value; }
        }
        /// <summary>
        /// Gets or sets the main frame for the delegate treatment
        /// </summary>
        public fr_Main Frame
        {
            get { return vFrame; }
            set { vFrame = value; }
        }
        #endregion
    }
}